@@ -1,4 +1,4 @@
-#  Copyright (c) 2020 Mira Geoscience Ltd.
+#  Copyright (c) 2021 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -15,12 +15,12 @@
 #  You should have received a copy of the GNU Lesser General Public License
 #  along with geoh5py.  If not, see <https://www.gnu.org/licenses/>.
 
-import numpy as np
 
-from .data import Data, DataType, PrimitiveTypeEnum
+from .data import DataType, PrimitiveTypeEnum
+from .numeric_data import NumericData
 
 
-class FloatData(Data):
+class FloatData(NumericData):
     """
     Data container for floats values
     """
@@ -32,29 +32,9 @@ class FloatData(Data):
     def primitive_type(cls) -> PrimitiveTypeEnum:
         return PrimitiveTypeEnum.FLOAT
 
-    @property
-    def values(self) -> np.ndarray:
+    @classmethod
+    def ndv(cls) -> float:
         """
-        :return: values: An array of float values
+        No-Data-Value
         """
-        if (getattr(self, "_values", None) is None) and self.existing_h5_entity:
-            self._values = self.check_vector_length(
-                self.workspace.fetch_values(self.uid)
-            )
-
-        return self._values
-
-    @values.setter
-    def values(self, values):
-        self.modified_attributes = "values"
-        self._values = self.check_vector_length(values)
-
-    def check_vector_length(self, values) -> np.ndarray:
-
-        full_vector = np.ones(self.n_values) * self.no_data_value
-        full_vector[: len(np.ravel(values))] = np.ravel(values)
-
-        return full_vector
-
-    def __call__(self):
-        return self.values
+        return 1.17549435e-38
